<?php

namespace Drupal\Tests\tag1quo\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests Tag1Quo configuration form.
 *
 * @group tag1quo
 */
class ConfigurationTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['tag1quo'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests configuration values are saved.
   */
  public function testConfigurationForm() {
    $user = $this->drupalCreateUser(['administer tag1quo']);
    $this->drupalLogin($user);

    $this->drupalGet('admin/config/development/tag1quo');
    $this->assertSession()->pageTextContainsOnce('No information will be sent to Tag1 Quo until you configure your token.');
    $this->assertSession()->checkboxNotChecked('debug[enabled]');

    // Assert the configuration form cannot be saved without token.
    $this->submitForm([], t('Save configuration'));
    $this->assertSession()->pageTextContainsOnce('No information will be sent to Tag1 Quo until you configure your token.');
    $this->assertSession()->pageTextContainsOnce('Token field is required.');

    // Assert the configuration form can be updated.
    $this->submitForm(['debug[enabled]' => TRUE, 'api[token]' => '123'], t('Save configuration'));
    $this->assertSession()->checkboxChecked('debug[enabled]');
    $this->assertSession()->pageTextNotContains('Token field is required.');
    $this->assertSession()->pageTextContainsOnce('No data was sent to Tag1 Quo. The API Token is invalid. It must be lower-cased alphanumeric string that is 32 characters in length.');

    // Set up a token override and assert it shows up.
    $settings['config']['tag1quo.settings']['api']['token'] = (object) [
      'value' => '123456',
      'required' => TRUE,
    ];
    $this->writeSettings($settings);
    $this->drupalGet('admin/config/development/tag1quo');
    $this->assertSession()->fieldValueEquals('api[token]', '123456');
  }

}
